// -*- C++ -*-
/*!
 * @file  HumanDetection_Camera.cpp 
 * @brief HumanDetection_Camera component
 * $Date$ 
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$ 
 */
#include "HumanDetection_Camera.h"

// Module specification
// <rtc-template block="module_spec">
static const char* humandetection_camera_spec[] =
  {
    "implementation_id", "HumanDetection_Camera",
    "type_name",         "HumanDetecCamera",
    "description",       "HumanDetection_Camera component",
    "version",           "1.0",
    "vendor",            "Nemoto,Mizoguchi Lab,Tokyo University of Science",
    "category",          "RobotModule",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.ColorSpace", "HSV",
    "conf.default.BackGround_Disp", "200",
    "conf.default.BackGround_XYZ", "2.000",
    "conf.default.Tilt_Rotation", "false",
    ""
  };
// </rtc-template>

HumanDetection_Camera::HumanDetection_Camera(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_ColorIn("Color", m_Color),
    m_DisparityIn("Disparity", m_Disparity),
    m_XYZDataIn("XYZData", m_XYZData),
    m_HeadAngleIn("HeadAngle", m_HeadAngle),
    m_PosOnImgOut("PosOnImg", m_PosOnImg),
    m_AreaOnImgOut("AreaOnImg", m_AreaOnImg),
    m_TargetPosOut("TargetPos", m_TargetPos),
    m_Target3DPosOut("Target3DPos", m_Target3DPos),
    m_TargetAngleOut("TargetAngle", m_TargetAngle),
    m_UpdateHSHistOut("UpdateHSHist", m_UpdateHSHist),
    m_CapPORTPort("CapPORT")

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
//  registerInPort("Color", m_ColorIn);
//  registerInPort("Disparity", m_DisparityIn);
//  registerInPort("XYZData", m_XYZDataIn);
//  registerInPort("HeadAngle", m_HeadAngleIn);

  // Set OutPort buffer
//  registerOutPort("PosOnImg", m_PosOnImgOut);
//  registerOutPort("AreaOnImg", m_AreaOnImgOut);
//  registerOutPort("TargetPos", m_TargetPosOut);
//  registerOutPort("Target3DPos", m_Target3DPosOut);
//  registerOutPort("TargetAngle", m_TargetAngleOut);
//  registerOutPort("UpdateHSHist", m_UpdateHSHistOut);

  // Set service provider to Ports

  // Set service consumers to Ports
//  m_CapPORTPort.registerConsumer("CameraInfo", "CameraInfo", m_CameraInfo);

  // Set CORBA Service Ports
//  registerPort(m_CapPORTPort);

  // </rtc-template>

}

HumanDetection_Camera::~HumanDetection_Camera()
{
}


RTC::ReturnCode_t HumanDetection_Camera::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("Color", m_ColorIn);
  addInPort("Disparity", m_DisparityIn);
  addInPort("XYZData", m_XYZDataIn);
  addInPort("HeadAngle", m_HeadAngleIn);

  // Set OutPort buffer
  addOutPort("PosOnImg", m_PosOnImgOut);
  addOutPort("AreaOnImg", m_AreaOnImgOut);
  addOutPort("TargetPos", m_TargetPosOut);
  addOutPort("Target3DPos", m_Target3DPosOut);
  addOutPort("TargetAngle", m_TargetAngleOut);
  addOutPort("UpdateHSHist", m_UpdateHSHistOut);

  // Set service provider to Ports

  // Set service consumers to Ports
  m_CapPORTPort.registerConsumer("CameraInfo", "CameraInfo", m_CameraInfo);

  // Set CORBA Service Ports
  registerPort(m_CapPORTPort);

    // <rtc-template block="bind_config">
    // Bind variables and configuration variable
    bindParameter("ColorSpace", m_ColorSpace, "HSV");
    bindParameter("BackGround_Disp", m_BackGround_Disp, "100");
    bindParameter("BackGround_XYZ", m_BackGround_XYZ, "3.000");
    bindParameter("Tilt_Rotation", m_Tilt_Rotation, "true");

    frame=cvCreateImage(cvSize(WIDTH,HEIGHT),8,3);
    disp=cvCreateImage(cvSize(WIDTH,HEIGHT),8,1);
    depth=cvCreateImage(cvSize(WIDTH,HEIGHT),8,1);
    target=cvCreateImage(cvSize(WIDTH,HEIGHT),8,3);
	HShist=cvCreateImage(cvSize(BIT8,BIT8),8,1);   


    xx= new double[WIDTH*HEIGHT];
    yy= new double[WIDTH*HEIGHT];
    zz= new double[WIDTH*HEIGHT];
    xx_F= new double[WIDTH*HEIGHT];
    yy_F= new double[WIDTH*HEIGHT];
    zz_F= new double[WIDTH*HEIGHT];
    
    aa= new int[WIDTH*HEIGHT];
    distant= new double[WIDTH*HEIGHT];

    // </rtc-template>
    return RTC::RTC_OK;
}


RTC::ReturnCode_t HumanDetection_Camera::onFinalize()
{
    delete [] xx;
    delete [] yy;
    delete [] zz;
    delete [] xx_F;
    delete [] yy_F;
    delete [] zz_F;
    
    delete [] aa;
    delete [] distant;

    cvReleaseImage(&frame);
    cvReleaseImage(&disp);
    cvReleaseImage(&depth);
    cvReleaseImage(&target);
    cvReleaseImage(&HShist);

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HumanDetection_Camera::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_Camera::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t HumanDetection_Camera::onActivated(RTC::UniqueId ec_id)
{
    VisionInit(m_ColorSpace,m_BackGround_XYZ);

    PortServiceList* portlist;
    portlist = this->get_ports();
    port_connect_check=new bool [portlist->length()];

    for (CORBA::ULong i(0), n(portlist->length()); i < n; ++i)
	{
	    PortService_ptr port;
	    port = (*portlist)[i];
	    cout << "PortNum"<<i<<":"<<port->get_port_profile()->name << "=";
	    if(port->get_port_profile()->connector_profiles.length()!=0){
		port_connect_check[i]=true;
		cout <<"Connect"<<endl;
	    }
	    else if(port->get_port_profile()->connector_profiles.length()==0){
		port_connect_check[i]=false;
		cout <<"DisConnect"<<endl;
	    }
	}
    cout <<"-----------------------"<<endl;
    PortService_ptr port;
    port = (*portlist)[10];//This port is ImageInformation Service Port
    if(port->get_port_profile()->connector_profiles.length()!=0){
	cout<<"Camera Information connect"<<endl;
	img_channels = m_CameraInfo->getImageChannels();
	img_h        = m_CameraInfo->getImageHeight();
	img_w        = m_CameraInfo->getImageWidth();
	img_size     = img_h*img_w*img_channels;
	if(img_h*img_w*img_channels==WIDTH*HEIGHT*3){
	    cout<<"OK:HumanDetection_Camera Component is enable"<<endl;
	}
	else if(img_h*img_w*img_channels!=WIDTH*HEIGHT*3){
	    cout<<"Error:Image Size is not 320×240"<<endl;	
	}
    }
    else if(port->get_port_profile()->connector_profiles.length()==0){
	cout<<"Camera Information Disconnect"<<endl;
    }
    cout <<"-----------------------"<<endl;



    ImgPos_uv[0]=0;
    ImgPos_uv[1]=0;

    m_TargetAngle.pan=0;
    m_TargetAngle.tilt=0;
    m_TargetPos.x=0;
    m_TargetPos.y=0;
    m_TargetPosOut.write();

    m_Target3DPos.x=0;
    m_Target3DPos.y=0;
    m_Target3DPos.z=0;
    m_TargetPosOut.write();

    m_TargetAngleOut.write();


    return RTC::RTC_OK;
}

RTC::ReturnCode_t HumanDetection_Camera::onDeactivated(RTC::UniqueId ec_id)
{
    m_TargetAngle.pan=0;
    m_TargetAngle.tilt=0;
    m_TargetPos.x=0;
    m_TargetPos.y=0;
    m_TargetPosOut.write();
    m_TargetAngleOut.write();

    m_Target3DPos.x=0;
    m_Target3DPos.y=0;
    m_Target3DPos.z=0;
    m_TargetPosOut.write();

    img_channels = 0;
    img_h        = 0;
    img_w        = 0;
    img_size     = 0;


    VisionEnd();
 
    delete [] port_connect_check;

    return RTC::RTC_OK;
}


RTC::ReturnCode_t HumanDetection_Camera::onExecute(RTC::UniqueId ec_id)
{

    static double pan=0,tilt=0;


    if(m_ColorIn.isNew()){
	m_ColorIn.read();
	if(m_Color.data.length()==WIDTH*HEIGHT*3){
	    memcpy(frame->imageData,(void *)&(m_Color.data[0]),WIDTH*HEIGHT*3);
	}
    }

    if(m_DisparityIn.isNew()){
	m_DisparityIn.read();
	if(m_Disparity.data.length()==WIDTH*HEIGHT){
	    memcpy(depth->imageData,(void *)&(m_Disparity.data[0]),WIDTH*HEIGHT);
	}
	for(int i=0;i<WIDTH*HEIGHT;i++){
	    if(depth->imageData[i]!=0) aa[i]=0;
	    else aa[i]=1;
	}
    }

    if(m_HeadAngleIn.isNew()){
	m_HeadAngleIn.read();
	pan=m_HeadAngle.pan;
	tilt=m_HeadAngle.tilt;
    }


    if(m_XYZDataIn.isNew()){
	m_XYZDataIn.read();
	if(m_XYZData.data.length()==WIDTH*HEIGHT){
	    for(int i=0;i<WIDTH*HEIGHT;i++){
		xx[i]=-m_XYZData.data[i].x/1000.0;
		yy[i]=m_XYZData.data[i].y/1000.0;
		zz[i]=m_XYZData.data[i].z/1000.0;
		if((xx[i]==0)&&(yy[i]==0)&&(zz[i]==0)) aa[i]=0;
		else aa[i]=1;
	    }
	}
			
    }


    Inputdis(zz,disp,distant,aa);

	BackClear(frame,distant,target,m_BackGround_XYZ);

    if((port_connect_check[2]==false)&&(port_connect_check[1]==true))
		BackClear_Disp(frame,depth,target,m_BackGround_Disp);


    Cover3d(aa,xx,yy,zz,xx_F,yy_F,zz_F);
    VisionInput(frame,target,distant);
    Vision3DInput(xx,yy,zz,aa);
    VisionLoop();
    VisionOutput(target_x,target_y,target_z);
    VisionImagePosOut(ImgPos_uv[0],ImgPos_uv[1]);
    VisionImageAreaOut(ImgArea_uv[0],ImgArea_uv[1],ImgArea_uv[2],ImgArea_uv[3]);


    if((target_z!=0)&&(target_z>=0)){
	PanTiltControl(pan,tilt,target_pan,target_tilt);
    }

    m_TargetPos.y=target_z*cos(-target_pan)-target_x*sin(-target_pan);
    m_TargetPos.x=target_z*sin(-target_pan)+target_x*cos(-target_pan);
    m_TargetPosOut.write();

    m_TargetAngle.pan=target_pan;

    //Enon_Head_Angle
    if(m_Tilt_Rotation==false){
	m_TargetAngle.tilt=0.0/180*PI;
    }

    //Segway_Head_Angle
    else {
	double length=target_z;
	m_TargetAngle.tilt=0.1309*length*length-0.6545*length+0.7854;
	if(m_TargetAngle.tilt>60.0*PI/180.0) m_TargetAngle.tilt=20.0*PI/180.0;
	if(length>2.0) m_TargetAngle.tilt=0;
	m_TargetAngle.tilt=15.0/180*PI;
    }

    m_TargetAngleOut.write();

	m_PosOnImg.data.length(2);
	for(int i=0;i<2;i++)    m_PosOnImg.data[i]=ImgPos_uv[i];
    m_PosOnImgOut.write();

	m_AreaOnImg.data.length(4);
	for(int i=0;i<4;i++)    m_AreaOnImg.data[i]=ImgArea_uv[i];
    m_AreaOnImgOut.write();

	m_Target3DPos.x=m_TargetPos.x;
	m_Target3DPos.y=m_TargetPos.y;
	m_Target3DPos.z=target_y;
	m_Target3DPosOut.write();


	//HShistOut(HShist);
    //m_UpdateHSHist.data.length(HShist->imageSize);
    //memcpy((void *)&(m_UpdateHSHist.data[0]),HShist->imageData,HShist->imageSize);
    //m_UpdateHSHistOut.write();



    return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t HumanDetection_Camera::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_Camera::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_Camera::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_Camera::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_Camera::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void HumanDetection_CameraInit(RTC::Manager* manager)
  {
    RTC::Properties profile(humandetection_camera_spec);
    manager->registerFactory(profile,
                             RTC::Create<HumanDetection_Camera>,
                             RTC::Delete<HumanDetection_Camera>);
  }
  
};



