// -*- C++ -*-
/*!
 * @file  HumanDetection_Camera.cpp
 * @brief HumanDetection_Camera component
 * $Date$ 
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$ 
 */

#ifndef HUMANDETECTION_CAMERA_H
#define HUMANDETECTION_CAMERA_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "MyDataTypeStub.h"

// </rtc-template>

#include<iostream>
#include <stdio.h>
#include <cv.h>
#include <cxcore.h>
#include <highgui.h>
#include "global.h"
#include "Vision.h"
using namespace std;


using namespace RTC;

class HumanDetection_Camera  : public RTC::DataFlowComponentBase
{
 public:
  HumanDetection_Camera(RTC::Manager* manager);
  ~HumanDetection_Camera();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
   virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  std::string m_ColorSpace;
  int m_BackGround_Disp;
  double m_BackGround_XYZ;
  bool m_Tilt_Rotation;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedOctetSeq m_Color;
  InPort<TimedOctetSeq> m_ColorIn;
  TimedOctetSeq m_Disparity;
  InPort<TimedOctetSeq> m_DisparityIn;
  TimedXYZLongSeq m_XYZData;
  InPort<TimedXYZLongSeq> m_XYZDataIn;
  TimedPanTilt m_HeadAngle;
  InPort<TimedPanTilt> m_HeadAngleIn;

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_PosOnImg;
  OutPort<TimedDoubleSeq> m_PosOnImgOut;
  TimedDoubleSeq m_AreaOnImg;
  OutPort<TimedDoubleSeq> m_AreaOnImgOut;
  TimedOdometry m_TargetPos;
  OutPort<TimedOdometry> m_TargetPosOut;
  TimedXYZDouble m_Target3DPos;
  OutPort<TimedXYZDouble> m_Target3DPosOut;
  TimedPanTilt m_TargetAngle;
  OutPort<TimedPanTilt> m_TargetAngleOut;
  TimedOctetSeq m_UpdateHSHist;
  OutPort<TimedOctetSeq> m_UpdateHSHistOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_CapPORTPort;

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  RTC::CorbaConsumer<CameraInfo> m_CameraInfo;

  // </rtc-template>

 private:
    IplImage* frame;  
    IplImage* disp;  
    IplImage* depth; 
    IplImage* target;
	IplImage* HShist;
    double target_x;
    double target_y;
    double target_z;

    double *xx;
    double *yy;
    double *zz;
    double *xx_F;
    double *yy_F;
    double *zz_F;
    int *aa;

    double *distant;
    double target_pan;
    double target_tilt;
	

  // image info
	int img_h, img_w, img_channels, img_size;

	double ImgPos_uv[2];
	double ImgArea_uv[4];

	bool *port_connect_check;
	//port_connect_check0:Color
	//port_connect_check1:Disparity
	//port_connect_check2:XYZData
	//port_connect_check3:HeadAngle
	//port_connect_check4:PosOnImg	
	//port_connect_check5:TargetPos
	//port_connect_check6:TargetAngle
	//port_connect_check7:CapPORT
};


extern "C"
{
  void HumanDetection_CameraInit(RTC::Manager* manager);
};

#endif // HUMANDETECTION_CAMERA_H

